local VizMarker = import('/lua/sim/VizMarker.lua').VizMarker

local OrigBeginSession = BeginSession
function BeginSession()
	LOG('Omega BeginSession(Original)')
	OrigBeginSession()
	LOG('Omega BeginSession(Omega)')

	OmegaMakeRadarMarkers()
	OmegaScanThread = ForkThread(OmegaScanFunction)
end



OmegaMakeRadarMarkers = function()

	local MapSizeX, MapSizeZ = GetMapSize()
	local MapRadius = MapSizeX * 1.5
	MapSizeX = (MapSizeX / 2)
	MapSizeZ = (MapSizeZ / 2)

	#loop through each army and give them full map radar coverage
	for BrainID, BrainRef in ArmyBrains do
		local VizData = {
			X = MapSizeX,
			Z = MapSizeZ,
			Radius = MapRadius,
			LifeTime = -1,
			Omni = false,
			Radar = true,
			Vision = false,
			Army = BrainRef:GetArmyIndex(),
		}

		LOG('Omega MakeRadarMarkers(' .. BrainRef:GetArmyIndex() .. ')')
		BrainRef.OmegaRadar = VizMarker(VizData)
		BrainRef.Trash:Add(BrainRef.OmegaRadar)
	end
end



#this function loops through all units to compare their locations and signature to all enemy radars
#if a unit is detected by ANY enemy radar than stealth is turned off and the unit becomes visible for ALL enemies (sorry)
#if the unit is not detected by ANY enemy radars than stealth is turned on and it becomes invisible
OmegaScanFunction = function()

	LOG('Omega ScanFunction()')

	local ScanDelay = 1
	local RadarList = {nil}
	local NewRadarList = {nil}

	while true do

		WaitSeconds(ScanDelay)

		AllUnitsList = OmegaGetAllUnits()		#get a list of all units in the map

		if AllUnitsList then
			for ID, PendingUnit in AllUnitsList do		#loop through each unit

#				if not (PendingUnit.OmegaHasBeenBuilt == 1) then
#					LOG('Omega CheckStatus: ' .. PendingUnit:GetUnitId() .. '( ' .. PendingUnit.OmegaHasBeenBuilt .. ' )')
#				end

#				LOG('Omega CheckStatus: ' .. PendingUnit:GetUnitId() .. '( ' .. PendingUnit.OmegaHasBeenBuilt .. ' / ' .. PendingUnit.OmegaHasIntelInit .. ' )')
#				LOG(repr(PendingUnit.IntelDisables))

				if (PendingUnit.OmegaHasBeenBuilt == 1) and not PendingUnit:IsDead() and PendingUnit:IsIntelEnabled('Radar') then	#check if this unit has radar
					table.insert(NewRadarList, PendingUnit)					#add this radar unit to the new radar list
				end

#				if PendingUnit:GetUnitId() == "xrb3301" then
#					LOG("Perimeter Monitor Bypass")
#				end

				if (PendingUnit.OmegaHasBeenBuilt == 1) and not PendingUnit.IsDead() then	#check that this is a valid unit
#				if (PendingUnit.OmegaHasBeenBuilt == 1) and not PendingUnit.IsDead() and not (PendingUnit:GetUnitId() == "xrb3301") then	#check that this is a valid unit

					local bDetected = false									#default to not detected
					local PendingSignature = PendingUnit:GetBlueprint().Intel.RadarSignature or 0.8 #0.75	#read radar signature or set default

					if PendingUnit:IsMoving() then	#if the pending unit is moving than modify its signature value
						PendingSignature = PendingSignature + (PendingUnit:GetBlueprint().Intel.RadarSignatureMovementMod or 0.2)
					end

					for RadarID, RadarUnit in RadarList do						#loop through all known radars
						if not RadarUnit:IsDead() and RadarUnit:IsIntelEnabled('Radar') then	#check that this radar is still valid
							if not IsAlly(RadarUnit:GetArmy(), PendingUnit:GetArmy()) then	#compare the radar team to the unit team

								local DistanceToUnit = VDist3(RadarUnit:GetPosition(), PendingUnit:GetPosition())		#check distance to radar
								local DistanceRatio = DistanceToUnit / (RadarUnit:GetBlueprint().Intel.RadarRadius or 50)	#compare distance to radar strength

								if DistanceRatio <= PendingSignature then		#the unit is close enough to be detected
									bDetected = true
									break
								end
							end
						else						#the radar was invalid (dead or off)
							table.remove(RadarUnit,RadarID)		#remove from radar list so we dont loop it with the rest of the units
						end
					end

					if bDetected then	#if this unit was detected by an enemy radar
						PendingUnit:DisableIntel('RadarStealth')	#turn off stealth and make it visible
						#PendingUnit:DisableUnitIntel('RadarStealth')	#turn off stealth and make it visible
					else			#if this unit wasn't detected
						PendingUnit:EnableIntel('RadarStealth')		#turn on stealth so its hidden
						#PendingUnit:EnableUnitIntel('RadarStealth')		#turn on stealth so its hidden
					end
				end
			end
		end

		RadarList = NewRadarList	#update the list of radars for the next loop
		NewRadarList = {nil}		#reset the temporary new radar list so we start fresh next loop
	end
end



#this function returns a list of all units in the map
OmegaGetAllUnits = function()

	local MapSizeX, MapSizeZ = GetMapSize()

	local RectX1 = -10
	local RectX2 = MapSizeX + 10
	local RectZ1 = -10
	local RectZ2 = MapSizeZ + 10

	return GetUnitsInRect(Rect(RectX1, RectZ1, RectX2, RectZ2))
end